const InstantiatedSDK = require("./instantiated_node.js");
const fs = require("fs");
const http = require("http");
const instance = new InstantiatedSDK();

function string_search_all(str, hay) {
    var pos=-1;
    for(var i=0; i<str.length; i++) {
        var g=str.substring(i, i+hay.length);

        if(g == hay) {
            pos=i;
        }
    }
    return pos;
}

function mimeTypeDetect(url) {
    var search_ext = url.substr(string_search_all(url, "."));

    switch(search_ext) {
        case ".png":
            return "image/png";
            break;
        case ".jpg":
            return "image/jpeg";
            break;
        case ".jpeg":
            return "image/jpeg";
            break;
        case ".gif":
            return "image/gif";
            break;
        case ".mp4":
            return "video/mp4";
            break;
        case ".mp3":
            return "audio/mpeg";
            break;
        case ".mpeg":
            return "video/mpeg";
            break;
        case ".webm":
            return "video/webm";
            break;
        case ".mkv":
            return "video/webm";
            break;
        case ".aac":
            return "audio/aac";
            break;
        case ".mid":
            return "audio/midi";
            break;
        case ".midi":
            return "audio/x-midi";
            break;
        case ".oga":
            return "audio/ogg";
            break;
        case ".ogg":
            return "audio/ogg";
            break;
        case ".opus":
            return "audio/ogg";
            break;
        case ".wav":
            return "audio/wav";
            break;
        case ".weba":
            return "audio/webm";
            break;
        default:
            return "application/octet-stream";
            break;
    }
}

function checkType(url, width = 256, type = "px", limit_width = false) {
    var mime = mimeTypeDetect(url).split("/");
    var ret = "";

    switch(mime[0]) {
        case "audio":
            if(limit_width) {
                ret = "<audio controls muted width=\""+width+type+"\"><source src=\""+url+"\" type=\""+mime[0]+"/"+mime[1]+"\"></audio>";
            } else {
                ret = "<audio controls muted><source src=\""+url+"\" type=\""+mime[0]+"/"+mime[1]+"\"></audio>";
            }
            break;
        case "video":
            if(limit_width) {
                ret = "<video controls muted width=\""+width+type+"\"><source src=\""+url+"\" type=\""+mime[0]+"/"+mime[1]+"\"></video>";
            } else {
                ret = "<video controls muted><source src=\""+url+"\" type=\""+mime[0]+"/"+mime[1]+"\"></video>";
            }
            break;
        case "image":
            if(limit_width) {
                ret = "<img src=\""+url+"\" width=\""+width+type+"\" />";
            } else {
                ret = "<img src=\""+url+"\" />";
            }
            break;
    }

    return ret;
}

/* 0 = INFO
1 = WARN
2 = ERROR */
function logTask(type, message) {
    switch(type) {
        case 0:
            console.log(Date(Date.now) + " [INFO]: "+message);
            break;
        case 1:
            console.log(Date(Date.now) + " [WARN]: "+message);
            break;
        case 2:
            console.log(Date(Date.now) + " [ERROR]: "+message);
            process.exit();
            break;
    }
}

let Profile = {
    Nickname: "???",
    Gender: "Male",
    ProfileImage: "https://example.com/image.png",
    BackgroundImage: "https://example.com/image.png",
    Age: 25,
    Bio: "This profile is currently being setup, please be patient with me as I get it set up.",
    Likes: ["Edit the likes in your settings."],
    Dislikes: ["Edit the dislikes in your settings."],
    Relationship: "Single",
    Work: "None",
    Status: "Edit your status in settings.",
    Website: "https://example.com/",
    WritePassword: "Please Write Your Password Here As It Will Be Used For Editing The Profile.",
    Feed: [],
    Media: {
        Pictures: [],
        Videos: [],
        Webpages: [],
        FamilyPhotos: []
    }
};

let Settings = {
    HTTP_IPADDRESS: "127.0.0.1",
    HTTP_PORT: 8005,
    SESSION: {
        ID: "YourSessionIDHere",
        PASSWORD: "PasswordHere",
        WHITELIST: {
            ENABLE: false,
            IPADDRESSES: []
        }
    },
    Bans: [],
    NUM_OF_ACCESS_ATTEMPT: 0 /* Immediately ban them for 5 minutes for incorrectly inputting the password. */
};

if(fs.existsSync("profile.json")) {
    Profile = JSON.parse(fs.readFileSync("profile.json"));
    logTask(0, "Loaded profile data.");
} else {
    fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
    logTask(0, "Created new profile with data.");
}

if(fs.existsSync("settings.json")) {
    Settings = JSON.parse(fs.readFileSync("settings.json"));
    logTask(0, "Loaded settings.");
} else {
    fs.writeFileSync("settings.json", JSON.stringify(Settings, null, '\t'));
    logTask(0, "Created new settings.");
}

let Clients = {};

instance.onOpen(() => {
    logTask(0, "Opening up your profile please wait...");

    if(Settings["SESSION"]["ID"] != "" || Settings["SESSION"]["ID"] != null) {
        instance.CreateServer(Settings["SESSION"]["ID"], "Profile Session", 50, Settings["SESSION"]["WHITELIST"]["ENABLE"], Settings["SESSION"]["PASSWORD"], false, false);
    } else {
        logTask(2, "Please provide a session id to your profile. It cannot be empty or invalid.");
    }

    setInterval(function() {
        instance.Ping();
    }, 15000);
});

instance.onMessage((msg) => {
    var id = JSON.parse(msg.toString("utf8"));

    switch(id["MSG"]) {
        case "SocketID":
            instance.SetSocketID(id["SocketID"]);
            break;
        case "created_session":
            instance.SetSessionID(id["SessionGUID"]);
            logTask(0, "Session created successfully. Visit https://instantiated.xyz/Samples/Profile/#"+id["SessionGUID"]+" to view your profile or send it to your friends.");

            for(var i=0; i<Settings["Bans"].length; i++) {
                instance.BanIP(Settings["Bans"][i]["ip"], null, null, Settings["Bans"][i]["reason"]);
                logTask(0, "Banning "+Settings["Bans"][i]["ip"] + " for "+Settings["Bans"][i]["reason"]+"...");
            }

            if(Settings["SESSION"]["WHITELIST"]["ENABLE"] == true) {
                for(var i=0; i<Settings["SESSION"]["WHITELIST"]["IPADDRESSES"].length; i++) {
                    instance.AddIPToWhitelist(Settings["SESSION"]["WHITELIST"]["IPADDRESSES"][i]);
                    logTask(0, "Adding "+Settings["SESSION"]["WHITELIST"]["IPADDRESSES"][i]);
                }
            }
            break;
        case "joined_session":
            Clients[id["SocketID"]] = {
                attempt_access: 0,
                ip: id["IPAddress"]
            };
            logTask(0, "Client "+id["SocketID"]+" has joined your session. IP Address : "+id["IPAddress"]);

            var copy = Object.assign({}, Profile);
            delete copy["WritePassword"];
            copy["Feed"] = [];

            for(var i=0; i<Profile["Feed"].length; i++) {
                copy["Feed"].push({
                    nick: Profile["Feed"][i]["nick"],
                    posted: Profile["Feed"][i]["posted"],
                    message: Profile["Feed"][i]["message"],
                    likes: Profile["Feed"][i]["likes"].length,
                    dislikes: Profile["Feed"][i]["dislikes"].length,
                    response: Profile["Feed"][i]["response"],
                    was_posted_by_profile: Profile["Feed"][i]["was_posted_by_profile"]
                });
            }

            instance.PrivateSend({
                id: "FetchProfile",
                ProfileInfo: copy
            }, id["SocketID"]);
            break;
        case "Disconnected":
            delete Clients[id["SocketID"]];

            logTask(0, "Client "+id["SocketID"]+" has lefted your session.");
            break;
        case "Tx":
            var tx=id["DataStream"];

            switch(tx["id"]) {
                case "PostComment":
                    var pos=Profile["Feed"].push({
                        nick: tx["nickname"],
                        posted: new Date(),
                        message: tx["message"],
                        likes: [],
                        dislikes: [],
                        response: null,
                        was_posted_by_profile: false
                    });
                    logTask(0, tx["nickname"]+" <"+id["IPAddress"]+"> said "+tx["message"]);
                    fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));

                    instance.Receive({
                        id: "PostedComment",
                        Feed: Profile["Feed"][pos - 1],
                        index: pos - 1
                    });
                    break;
                case "LikePost":
                    if(Profile["Feed"][tx["postid"]] != null) {
                        if(Profile["Feed"][tx["postid"]]["likes"].indexOf(id["IPAddress"]) == -1 && Profile["Feed"][tx["postid"]]["dislikes"].indexOf(id["IPAddress"]) == -1) {
                            var pos=Profile["Feed"][tx["postid"]]["likes"].push(id["IPAddress"]);
                            fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));

                            instance.Receive({
                                id: "LikedPost",
                                commentId: tx["postid"],
                                num: Profile["Feed"][tx["postid"]]["likes"].length
                            });
                        }
                    }
                    break;
                case "DislikePost":
                    if(Profile["Feed"][tx["postid"]] != null) {
                        if(Profile["Feed"][tx["postid"]]["dislikes"].indexOf(id["IPAddress"]) == -1 && Profile["Feed"][tx["postid"]]["likes"].indexOf(id["IPAddress"]) == -1) {
                            var pos=Profile["Feed"][tx["postid"]]["dislikes"].push(id["IPAddress"]);
                            fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));

                            instance.Receive({
                                id: "DislikedPost",
                                commentId: tx["postid"],
                                num: Profile["Feed"][tx["postid"]]["dislikes"].length
                            });
                        }
                    }
                    break;
                case "EditProfile":
                    if(Profile["WritePassword"] == tx["write_passwd"]) {
                        if(tx["key"] in Profile) {
                            Profile[tx["key"]] = tx["value"];
                            fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                            
                            instance.PrivateSend({
                                id: "ProfileEdited",
                                NewProfile: Profile
                            }, id["SocketID"]);
                        }
                    } else {
                        Clients["attempt_access"]++;
                        if(Clients["attempt_access"] >= Settings["NUM_OF_ACCESS_ATTEMPT"]) {
                            instance.BanIP(Clients["ip"], id["SocketID"], 60*5, "[Auto Ban] Attempted to access the write permission to my profile.");
                        }
                    }
                    break;
            }
            break;
    }
});

const httpServer = http.createServer((req, res) => {
    var q = new URL(req.url, `http://${req.headers.host}`);
    var query = q.searchParams;
    var pathname = q.pathname;
    var html=``;
    var headerLink = `<a href="/profile">Manage Profile</a> | <a href="/comments">Manage Comments</a> | <a href="/bans">Manage Bans</a> | <a href="/media">Upload/Edit Media Files</a>`

    switch(pathname) {
        case "/media":
            if(query.get("cmd") == null) {
                html+=`<html><head><title>Your Profile - Upload Media Files</title></head><body><center><h1>Welcome to your profile management.</h1>`+headerLink+`</center>`;
                html+=`<h1>List/Create Folder</h1>`;
                html+=`<form action="/media" method="GET"><input type="hidden" name="cmd" value="createfolder" />Folder Name: <input type="text" name="foldername" /><input type="submit" /></form><br>`;
                html+=`<form action="/media" method="GET"><input type="hidden" name="cmd" value="uploadfile" /><input type="hidden" id="fileOutput" name="fileInput" value="" />`;
                html+=`Select Folder: <select name="folderId">`;
                for(md in Profile["Media"]) {
                    html+=`<option value="`+md.replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`">`+md.replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`</option>`;
                }
                html+=`</select><br>`;
                html+=`Upload Media File: <input type="file" id="uploadFile" onchange="onUpload();" /><br>`;
                html+=`Title Media: <input type="text" name="mediaName" /><br>`;
                html+=`<input type="submit" /></form><br>`;
                
                for(md in Profile["Media"]) {
                    html+=`<b>`+md.replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`</b> - <a href="/media?folder=`+md.replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`&cmd=deletefolder">Delete Folder</a><ul>`;
                    for(var i=0; i<Profile["Media"][md].length; i++) {
                        var parse = Profile["Media"][md][i];

                        html+=`<li>`+checkType(parse["url"].replaceAll('<', '&lt;').replaceAll('>', '&gt;'), 256, "px", true)+`<br><b>`+parse["name"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`</b> - <a href="/media?id=`+i+`&folder=`+md.replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`&cmd=delete">Delete Media</a></li>`;
                    }
                    html+=`</ul>`;
                }
                html+=`<script>
function onUpload() {
    var http = new XMLHttpRequest();
    var form = new FormData();

    form.append("fileName", uploadFile.files[0]);
    http.open("POST", "https://instantiated.xyz/upload.php", true);
    http.onload = function() {
        var j = JSON.parse(this.response);

        fileOutput.value = j["url"];
    }
    http.send(form);
}</script></body></html>`;
            } else {
                switch(query.get("cmd")) {
                    case "createfolder":
                        Profile["Media"][query.get("foldername")] = [];
                        fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                        html = "<script>window.location.href=\"/media\";</script>";
                        break;
                    case "uploadfile":
                        Profile["Media"][query.get("folderId")].push({url: query.get("fileInput"), name: query.get("mediaName")});
                        fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                        html = "<script>window.location.href=\"/media\";</script>";
                        break;
                    case "deletefolder":
                        if(query.get("accept") != null) {
                            delete Profile["Media"][query.get("folder")];
                            fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                            html = "<script>window.location.href=\"/media\";</script>";
                        } else {
                            html = "<i>Do you really want to delete <b>"+query.get("folder")+"</b>?</i><br>";
                            html += "<a href=\"/media?folder="+query.get("folder")+"&cmd=deletefolder&accept=true\">Delete Folder</a> or <a href=\"/media\">DO NOT DELETE, GET ME BACK!</a>";
                        }
                        break;
                    case "delete":
                        if(query.get("accept") != null) {
                            Profile["Media"][query.get("folder")].splice(parseInt(query.get("id")), 1);
                            fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                            html = "<script>window.location.href=\"/media\";</script>";
                        } else {
                            var p=Profile["Media"][query.get("folder")][parseInt(query.get("id"))];
                            html = `<i>Do you really want to delete <b>`+p["name"] + ` in `+query.get("folder")+`</b>?<br>`;
                            html += `<a href="/media?folder=`+query.get("folder")+`&id=`+query.get("id")+`&cmd=delete&accept=true">Delete media</a> or <a href="/media">DO NOT DELETE, GET ME BACK!</a>`;
                        }
                        break;
                }
            }
            break;
        case "/profile":
            if(query.get("edit") == null) {
                html+=`<html>
    <head>
        <title>Your Profile - Editing Profile</title>
    </head>
    <body>
        <center><h1>Welcome to your profile management.</h1>
        `+headerLink+`</center>
        <img src="`+Profile["ProfileImage"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" width="128px" /><br>
        <form action="/profile" method="GET">
            Nickname : <input type="text" name="nickname" value="`+Profile["Nickname"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" /><br>
            Gender : <select name="gender"><option value=""></option><option value="Male">Male &#9794;</option><option value="Female">Female &#9792;</option></select><br>
            Profile Image : <input type="hidden" name="profileImage" id="setImage" value="`+Profile["ProfileImage"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" /><input type="file" id="uploadFile" onchange="_uploadFile();" accepts="image/*" /><br>
            Background Image : <input type="hidden" name="backgroundImage" id="setBackground" value="`+Profile["BackgroundImage"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" /><input type="file" id="uploadFileBg" onchange="_uploadFileBg();" accepts="image/*" /><br>
            Age : <input type="number" min="18" max="100" name="age" value="`+Profile["Age"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" /><br>
            Bio : <br>
            <textarea name="bio">`+Profile["Bio"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`</textarea><br>
            Likes (Seperated by ", ") : <input type="text" name="likes" value="`;
            for(var i=0; i<Profile["Likes"].length; i++) {
                if(i==0) {
                    html+=Profile["Likes"][i].replaceAll('<', '&lt;').replaceAll('>', '&gt;');
                } else {
                    html+=`, `+Profile["Likes"][i].replaceAll('<', '&lt;').replaceAll('>', '&gt;');
                }
            }
            html+=`" /><br>
            Dislikes (Same with Likes) : <input type="text" name="dislikes" value="`;
            for(var i=0; i<Profile["Dislikes"].length; i++) {
                if(i==0) {
                    html+=Profile["Dislikes"][i].replaceAll('<', '&lt;').replaceAll('>', '&gt;');
                } else {
                    html+=`, `+Profile["Dislikes"][i].replaceAll('<', '&lt;').replaceAll('>', '&gt;');
                }
            }
            html+=`" /><br>
            Relationship : <select name="relationship"><option value=""></option><option value="Divorced">Divorced</option><option value="Married">Married</option><option value="Taken on a Date">Taken on a Date</option><option value="Single">Single</option></select><br>
            Work : <input type="text" name="work" value="`+Profile["Work"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" /><br>
            Status : <input type="text" name="status" value="`+Profile["Status"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" /><br>
            Website : <input type="text" name="website" value="`+Profile["Website"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`" /><br>
            Write Password Settings<br>
            <input type="password" name="write_password" /><br>
            <input type="submit" name="edit" value="Write" />
        </form>
        <script>
            function _uploadFileBg() {
                var http = new XMLHttpRequest();
                var form = new FormData();

                form.append("fileName", uploadFileBg.files[0]);

                http.open("POST", "https://instantiated.xyz/upload.php", true);
                http.onload = function() {
                    var id = JSON.parse(this.response);

                    setBackground.value = id["url"];
                }
                http.send(form);
            }

            function _uploadFile() {
                var http = new XMLHttpRequest();
                var form = new FormData();

                form.append("fileName", uploadFile.files[0]);

                http.open("POST", "https://instantiated.xyz/upload.php", true);
                http.onload = function() {
                    var id = JSON.parse(this.response);

                    setImage.value = id["url"];
                }
                http.send(form);
            }
        </script>
    </body>
</html>`;
            } else {
                if(query.get("nickname") != "") {Profile["Nickname"] = query.get("nickname");}
                if(query.get("gender") != "") {Profile["Gender"] = query.get("gender");}
                if(query.get("profileImage") != "") {Profile["ProfileImage"] = query.get("profileImage");}
                if(query.get("backgroundImage") != "") {Profile["BackgroundImage"] = query.get("backgroundImage");}
                if(query.get("age") != "") {Profile["Age"] = query.get("age");}
                if(query.get("bio") != "") {Profile["Bio"] = query.get("bio");}
                if(query.get("likes") != "") {
                    var likes = query.get("likes").split(", ");
                    Profile["Likes"] = [];
                    for(var i=0; i<likes.length; i++) {
                        Profile["Likes"].push(likes[i]);
                    }
                }
                if(query.get("dislikes") != "") {
                    var dislikes = query.get("dislikes").split(", ");
                    Profile["Dislikes"] = [];
                    for(var i=0; i<dislikes.length; i++) {
                        Profile["Dislikes"].push(dislikes[i]);
                    }
                }
                if(query.get("relationship") != "") {Profile["Relationship"] = query.get("relationship");}
                if(query.get("work") != "") {Profile["Work"] = query.get("work");}
                if(query.get("status") != "") {Profile["Status"] = query.get("status");}
                if(query.get("website") != "") {Profile["Website"] = query.get("website");}
                if(query.get("write_password") != "") {Profile["WritePassword"] = query.get("write_password");}

                var copy = Object.assign({}, Profile);
                delete copy["WritePassword"];
                delete copy["Feed"];

                instance.Receive({
                    id: "UpdateProfile",
                    ProfileInfo: copy
                });
                
                fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                html = "<script>window.location.href=\"/profile\";</script>";
            }
            break;
        case "/comments":
            switch(query.get("cmd")) {
                case "post":
                    var pos = Profile["Feed"].push({
                        nick: Profile["Nickname"],
                        posted: new Date(),
                        message: query.get("message"),
                        likes: [],
                        dislikes: [],
                        response: null,
                        was_posted_by_profile: true
                    });

                    instance.Receive({
                        id: "PostedComment",
                        Feed: Profile["Feed"][pos - 1],
                        index: pos - 1
                    });

                    fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                    html = "<script>window.location.href=\"/comments\";</script>";
                    break;
                case "delete":
                    Profile["Feed"].splice(parseInt(query.get("id")), 1);

                    fs.writeFileSync("profile.json", JSON.stringify(Profile, null, '\t'));
                    html = "<script>window.location.href=\"/comments\";</script>";
                    break;
                default:
                    html+=`<html>
    <head>
        <title>Your Profile - Comments</title>
    </head>
    <body>
        <center><h1>Welcome to your profile management.</h1>
        `+headerLink+`</center>
        <ul>`;
                    for(var i=0; i<Profile["Feed"].length; i++) {
                        html+=`<li><b>`+Profile["Feed"][i]["nick"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`</b> Posted on `+Profile["Feed"][i]["posted"]+` said<br><pre>`+Profile["Feed"][i]["message"].replaceAll('<', '&lt;').replaceAll('>', '&gt;')+`</pre><br><a href="/comments?cmd=delete&id=`+i+`">Delete Comment</a></li>`;
                    }
                    html+=`</ul>
        <form action="/comments" method="GET">
            <input type = "hidden" name="cmd" value="post" />
            <textarea name = "message"></textarea><br>
            <input type = "submit" /></form><body></html>`;
                    break;
            }
            break;
        case "/bans":
            switch(query.get("cmd")) {
                case "unban":
                    instance.UnbanIP(Settings["Bans"][parseInt(query.get("id"))]["ip"]);
                    Settings["Bans"].splice(parseInt(query.get("id")), 1);
                    fs.writeFileSync("settings.json", JSON.stringify(Settings, null, '\t'));
                    html = "<script>window.location.href=\"/bans\";</script>";
                    break;
                case "ban":
                    if(query.get("duration") == "permanent") {
                        Settings["Bans"].push({
                            ip: query.get("ipaddr"),
                            reason: query.get("reason")
                        });
                        fs.writeFileSync("settings.json", JSON.stringify(Settings, null, '\t'));

                        for(cl in Clients) {
                            if(Clients[cl]["ip"] == query.get("ipaddr")) {
                                console.log(cl);
                                instance.BanIP(query.get("ipaddr"), parseInt(cl), null, query.get("reason"));
                            }
                        }
                    } else {
                        for(cl in Clients) {
                            if(Clients[cl]["ip"] == query.get("ipaddr")) {
                                instance.BanIP(query.get("ipaddr"), parseInt(cl), parseInt(query.get("duration")),  query.get("reason"));
                            }
                        }
                    }
                    html = "<script>window.location.href=\"/bans\";</script>";
                    break;
                default:
                    html+=`<html>
    <head>
        <title>Your Profile - Bans</title>
    </head>
    <body>
        <center><h1>Welcome to your profile management.</h1>
        `+headerLink+`</center>
        <ul>`;
                    for(var i=0; i<Settings["Bans"].length; i++) {
                        html+=`<li>`+Settings["Bans"][i]["ip"]+` was banned for `+Settings["Bans"][i]["reason"]+` - <a href="/bans?cmd=unban&id=`+i+`">Unban IP</a></li>`;
                    }
                    html+=`</ul><form action="/bans" method="GET">
    <input type="hidden" name="cmd" value="ban" />IP : <input type="text" name="ipaddr" /><br>
    Reason : <input type="text" name="reason" /><br>
    Duration : <select name="duration">
        <option value="86400">1 Day</option>
        <option value="2592000">30 Days</option>
        <option value="permanent">Permanent Ban (Gets Saved)</option>
    </select><br>
    <input type="submit" /></form><body></html>`;
                    break;
            }
            break;
        default:
            html+=`<html>
    <head>
        <title>Your Profile - Main</title>
    </head>
    <body>
        <center><h1>Welcome to your profile management.</h1>
        `+headerLink+`</center>
        <b>Have someone scan your qrcode for them to connect.</b><br>
        <img src="https://qrcode.tec-it.com/API/QRCode?data=https%3A%2F%2Finstantiated.xyz%2FSamples%2FProfile%2F%23`+Settings["SESSION"]["ID"]+`" width="256px" />
    </body>
</html>`;
            break;
    }
    res.writeHead(200, { 'Content-Type': 'text/html' });
    res.end(html);
}).listen(Settings["HTTP_PORT"], Settings["HTTP_IPADDRESS"]);